@@ -11,29 +11,35 @@ import os
 import tempfile
 import shlex
 import subprocess
-from sys import exit
+from sys import exit, argv
 
 # Python < 3.x
 try:
+    import Tix
     import Tkinter as tk
     from ScrolledText import ScrolledText
-    from tkFileDialog import \
-        (asksaveasfilename, askopenfilename)
     from tkMessageBox import showinfo, showerror
+    from tkFileDialog import (
+      asksaveasfilename, askopenfilename
+      )
+
 # Python 3.x
 except ImportError:
     import tkinter as tk
+    import tkinter.tix as Tix
     from tkinter.scrolledtext import ScrolledText
-    from tkinter.filedialog import \
-        (asksaveasfilename, askopenfilename)
     from tkinter.messagebox import showinfo, showerror
+    from tkinter.filedialog import (
+      asksaveasfilename, askopenfilename
+      )
 
 from libtovid import cli
 from libtovid.metagui.widget import Widget
 from libtovid.metagui.panel import Panel, Tabs
-from libtovid.metagui.support import \
-    (ConfigWindow, Style, ensure_type, askyesno)
 from libtovid.metagui.control import Control, NoSuchControl
+from libtovid.metagui.support import (
+  ConfigWindow, Style, ensure_type, askyesno, get_photo_image, show_icons
+  )
 
 
 DEFAULT_CONFIG = os.path.expanduser('~/.metagui/config')
@@ -60,25 +66,25 @@ class Executor (Widget):
         # Log output text area
         self.text = ScrolledText(self, width=1, height=1,
             highlightbackground='gray', highlightcolor='gray', relief='groove',
-            font=('Courier', 12, 'normal'))
+            font=('Nimbus Sans L', 12, 'normal'))
         self.text.pack(fill='both', expand=True)
         # Bottom frame to hold the next four widgets
-        frame = tk.Frame(self)
+        self.frame = tk.Frame(self)
         # Text area for stdin input to the program
-        label = tk.Label(frame, text="Input:")
+        label = tk.Label(self.frame, text="Input:")
         label.pack(side='left')
-        self.stdin_text = tk.Entry(frame)
+        self.stdin_text = tk.Entry(self.frame)
         self.stdin_text.pack(side='left', fill='x', expand=True)
         self.stdin_text.bind('<Return>', self.send_stdin)
         # Button to stop the process
-        self.kill_button = tk.Button(frame, text="Kill", command=self.kill)
+        self.kill_button = tk.Button(self.frame, text="Kill", command=self.kill)
         self.kill_button.pack(side='left')
         # Button to save log output
-        self.save_button = tk.Button(frame, text="Save log",
+        self.save_button = tk.Button(self.frame, text="Save log",
                                      command=self.save_log)
         self.save_button.pack(side='left')
         # Pack the bottom frame
-        frame.pack(anchor='nw', fill='x')
+        self.frame.pack(anchor='nw', fill='x')
         # Disable stdin box and kill button until execution starts
         self.stdin_text.config(state='disabled')
         self.kill_button.config(state='disabled')
@@ -89,7 +95,9 @@ class Executor (Widget):
         """
         text = self.stdin_text.get()
         # Write the entered text to the command's stdin pipe
-        self.command.proc.stdin.write(text + '\n')
+        #self.command.proc.stdin.write(text + '\n')
+        # python 3 compatibility
+        self.command.proc.stdin.write(bytearray(text + "\n", "ascii"))
         self.command.proc.stdin.flush()
         # Show what was typed in the log window
         self.write(text)
@@ -97,17 +105,16 @@ class Executor (Widget):
         self.stdin_text.delete(0, 'end')
 
 
-    def execute(self, command, callback=lambda x:x):
-        """Execute the given command, and call the given callback when done.
+    def execute(self, command, callback=None):
+        """Execute the given `~libtovid.cli.Command`, and call the given callback when done.
         """
         if not isinstance(command, cli.Command):
             raise TypeError("execute() requires a Command instance.")
-        else:
-            self.command = command
-        if not hasattr(callback, '__call__'):
-            raise TypeError("execute() callback must be a function")
-        else:
-            self.callback = callback
+        if not callable(callback):
+            raise TypeError("execute() callback must be callable")
+
+        self.command = command
+        self.callback = callback
 
         # Temporary file to hold stdout/stderr from command
         name = self.command.program
@@ -145,12 +152,15 @@ class Executor (Widget):
         """Poll for process completion, and update the output window.
         """
         # Read from output file and print to log window
-        data = self.outfile.read()
-        if data:
-            # Split on newlines (but not on \r)
-            lines = data.split('\n')
-            for line in lines:
-                self.write(line)
+        try:
+            data = self.outfile.read()
+            if data:
+                # Split on newlines (but not on \r)
+                lines = data.split('\n')
+                for line in lines:
+                    self.write(line)
+        except ValueError:
+            pass
 
         # Stop if command is done, or poll again
         if self.command.done():
@@ -172,7 +182,7 @@ class Executor (Widget):
 
     def write(self, line):
         """Write a line of text to the end of the log.
-        If the line contains '\r', overwrite the current line.
+        If the line contains ``\\r``, overwrite the current line.
         """
         if '\r' in line:
             curline = self.text.index('end-1c linestart')
@@ -205,6 +215,7 @@ class Executor (Widget):
             self.notify("Output saved to '%s'" % filename)
 
 
+from textwrap import dedent
 class Application (Widget):
     """Graphical frontend for a command-line program
     """
@@ -241,10 +252,14 @@ class Application (Widget):
         """Draw the Application in the given master.
         """
         Widget.draw(self, master)
+        self.master = master
         # Draw all panels as tabs
         self.tabs = Tabs('', *self.panels)
         self.tabs.draw(self)
         self.tabs.pack(anchor='n', fill='both', expand=True)
+        # need to wait until draw is called before setting this variable
+        self.script = tk.StringVar()
+        self.script.set('')
 
 
     def draw_toolbar(self, config_function, exit_function):
@@ -279,7 +294,7 @@ class Application (Widget):
             save_button.pack(anchor='w', side='left', fill='x', expand=True)
             save_button.config(command=self.save_exit, text='Save to wizard')
             run_button.pack_forget()
-#            exit_button.pack_forget()
+            #exit_button.pack_forget()
 
 
     def get_args(self):
@@ -291,6 +306,15 @@ class Application (Widget):
         return args
 
 
+    def set_args(self, args):
+        """Load application settings from a list of command-line arguments.
+        The list of args is modified in-place; if all args are successfully
+        parsed and loaded, the list will be empty when this method returns.
+        """
+        for panel in self.panels:
+            panel.set_args(args)
+
+
     def execute(self):
         """Run the program with all the supplied options.
         """
@@ -314,6 +338,13 @@ class Application (Widget):
             self.executor.notify("Cancelled.")
             self.toolbar.enable()
 
+    def set_scriptname(self, name):
+        """Set script filename.  Called externally, this sets the variable
+        that used used for save_exit() as well as the 'initial file' for
+        prompt_save_script().
+        """
+        self.script.set(name)
+
 
     def prompt_save_script(self):
         """Prompt for a script filename, then save the current
@@ -322,7 +353,8 @@ class Application (Widget):
         # TODO: Make initialfile same as last saved script, if it exists
         filename = asksaveasfilename(parent=self,
             title="Select a filename for the script",
-            initialfile='%s_commands.bash' % self.program)
+            #initialfile='%s_commands.bash' % self.program)
+            initialfile=self.script.get() or '%s_commands.bash' % self.program)
 
         if not filename:
             return
@@ -333,18 +365,34 @@ class Application (Widget):
             showerror(title="Error", message="Failed to save '%s'" % filename)
             raise
         else:
+            saved_script = os.path.split(filename)[1]
             showinfo(title="Script saved", message="Saved '%s'" % filename)
 
+
     def save_exit(self):
-        filename = os.getcwd() + '/todisc_commands.bash'
+        """Save current command to script and exit, without prompt
+        """
+        # hack for titleset wizard to get gui's screen position
+        from sys import stderr
+        stderr.write("%s %+d%+d" 
+          %('gui position', self._root().winfo_x(), self._root().winfo_y()))
+
+        default = os.getcwd() + os.path.sep + '%s_commands.bash' % self.program
+        filename = self.script.get() or default
         self.save_script(filename)
-        # exit code borrowed from apache so wizard knows it has a file to read
-        exit(200)
+        exit()
+
 
     def prompt_load_script(self):
-        """Prompt for a script filename, then load current Control settings
-        from that file.
+        """Prompt for script filename; reload gui with current Control settings
         """
+        #"""Prompt for a script filename, then load current Control settings
+        #from that file.
+        #"""
+        # Hack: This is broken (commented section), presumably because of
+        # load_args limitations. This hack fixes it for libtovid, but does not
+        # belong in metagui.  So refactor this when load_args is fixed.
+        # see https://github.com/tovid-suite/tovid/issues/121
         filename = askopenfilename(parent=self,
             title="Select a shell script or text file to load",
             filetypes=[('Shell scripts', '*.sh *.bash'),
@@ -352,15 +400,23 @@ class Application (Widget):
 
         if not filename:
             return
-
+        # begin Hack
+        geometry = self._root().winfo_x(), self._root().winfo_y()
+        command = argv[0]
         try:
-            self.reset()
-            self.load_script(filename)
+            cmd = command, '--position', '+%s+%s' %geometry, filename
+            os.execlp(cmd[0], *cmd)
         except:
-            showerror(title="Error", message="Failed to load '%s'" % filename)
+            showerror(title="Error", message="Failed load '%s'" %filename)
             raise
-        else:
-            showinfo(title="Script loaded", message="Loaded '%s'" % filename)
+        #try:
+        #    self.reset()
+        #    self.load_script(filename)
+        #except:
+        #    showerror(title="Error", message="Failed to load '%s')
+        #    raise
+        #else:
+        #    showinfo(title="Script loaded", message="Loaded '%s'" % filename)
 
 
     def save_script(self, filename):
@@ -382,28 +438,32 @@ class Application (Widget):
 
 
     def load_script(self, filename):
-        """Load current Control settings from a text file.
-        """
+        # this is now disabled and its functionality passed to guis.helpers.py
+        # see https://github.com/tovid-suite/tovid/issues/121
+        pass
+        #"""Load current Control settings from a text file.
+        #"""
         # Read lines from the file and reassemble the command
-        command = ''
-        for line in open(filename, 'r'):
-            line = line.strip()
+        #command = ''
+        #for line in open(filename, 'r'):
+        #    line = line.strip()
             # Discard comment lines and PATH assignment
-            if line and not (line.startswith('#') or line.startswith('PATH=')):
-                command += line.rstrip('\\')
+        #    if line and not (line.startswith('#') or line.startswith('PATH=')):
+        #        command += line.rstrip('\\')
         # Split the full command into arguments, according to shell syntax
-        args = shlex.split(command)
+        #args = shlex.split(command)
 
         # Ensure the expected program is being run
-        program = args.pop(0)
-        if program != self.program:
-            raise ValueError("This script runs '%s', expected '%s'" %
-                  (program, self.program))
+        #program = args.pop(0)
+        #if program != self.program:
+        #    raise ValueError("This script runs '%s', expected '%s'" %
+        #          (program, self.program))
         # Load the rest of the arguments
-        self.load_args(args)
-        print("Done reading '%s'" % filename)
+        #self.load_args(args)
+        #print("Done reading '%s'" % filename)
 
 
+    # TODO: Relocate all this stuff into Control and its subclasses
     def load_args(self, args):
         """Load settings from a list of command-line arguments.
         """
@@ -443,8 +503,10 @@ class Application (Widget):
                 control.set(control.vartype(value))
 
 
-
-class GUI (tk.Tk):
+# This uses Tix.Tk as a base class, to allow Tix widgets within
+# (since for some reason a Tix.ComboBox doesn't like to be inside
+# a Tkinter.Tk root window)
+class GUI (Tix.Tk):
     """GUI with one or more Applications
     """
     def __init__(self, title, width, height, application, **kwargs):
@@ -461,26 +523,33 @@ class GUI (tk.Tk):
 
         Keywords arguments accepted:
 
-            inifile:      Name of an .ini-formatted file with GUI configuration
+            inifile
+                Name of an .ini-formatted file with GUI configuration
+            icon
+                Full path to an image to use as the titlebar icon
+            position
+                position on screen: '+X+Y'
         """
-        tk.Tk.__init__(self)
+        Tix.Tk.__init__(self)
 
         # Ensure that one or more Application instances were provided
         ensure_type("GUI needs Application", Application, application)
         self.application = application
 
+        # Get keyword arguments
+        self.inifile = kwargs.get('inifile', DEFAULT_CONFIG)
+        self.icon_file = kwargs.get('icon', None)
+        self.position = kwargs.get('position', '')
+
         # Set main window attributes
-        self.geometry("%dx%d" % (width, height))
+        self.geometry("%dx%d%s" % (width, height, self.position))
         self.title(title)
         self.width = width
         self.height = height
 
-        # Get style configuration from INI file
-        self.inifile = kwargs.get('inifile', DEFAULT_CONFIG)
-
         self.style = Style()
         if os.path.exists(self.inifile):
-            print("Loading style from config file: '%s'" % self.inifile)
+            print("Loading style from the config file: '%s'" % self.inifile)
             self.style.load(self.inifile)
         else:
             print("Creating config file: '%s'" % self.inifile)
@@ -496,11 +565,14 @@ class GUI (tk.Tk):
         self.bind('<Control-q>', self.confirm_exit)
 
 
-    def run(self, args=None):
+    def run(self, args=None, script=''):
         """Run the GUI and enter the main event handler.
         This function does not return until the GUI is closed.
+        args: arguments to the programe being run
+        script: a filename to save the GUI command output to
         """
         self.draw()
+        
         if args:
             # If first argument looks like a filename, load
             # a script from that file
@@ -518,6 +590,9 @@ class GUI (tk.Tk):
             # If more args remain, load those too
             if args:
                 self.application.load_args(args)
+            # if a script/project name was provided, set it in 'application'
+        if script:
+            self.application.set_scriptname(script)
 
         # Enter the main event handler
         self.mainloop()
@@ -535,6 +610,9 @@ class GUI (tk.Tk):
         self.application.pack(anchor='n', fill='both', expand=True)
         # Draw the toolbar
         self.application.draw_toolbar(self.show_config, self.confirm_exit)
+        # Set the application icon, if provided
+        if self.icon_file:
+            show_icons(self, self.icon_file)
 
 
     def show_config(self):
@@ -555,6 +633,11 @@ class GUI (tk.Tk):
     def confirm_exit(self, event=None):
         """Exit the GUI, with confirmation prompt.
         """
+        # Hack to sync titleset wizard's position with exiting GUI's position
+        if os.getenv('METAGUI_WIZARD'):
+            from sys import stderr
+            stderr.write("%s %+d%+d" 
+            %('gui position', self._root().winfo_x(), self._root().winfo_y()))
         if askyesno(message="Exit?"):
             self.quit()
 
