@@ -1,10 +1,12 @@
-"""Supporting classes for metagui"""
+"""Supporting classes for metagui
+"""
 
 __all__ = [
     # Functions
     'exit_with_traceback',
     'ensure_type',
     'divide_list',
+    'get_photo_image',
     # Others
     'ScrollList',
     'DragList',
@@ -21,6 +23,7 @@ import sys
 import traceback
 import math
 import base64
+import re
 
 # Python < 3.x
 try:
@@ -28,12 +31,27 @@ try:
     from tkSimpleDialog import Dialog
     import tkMessageBox
     from ConfigParser import ConfigParser
+    import Tix
+    import tkFont
+
 # Python 3.x
 except ImportError:
     import tkinter as tk
     from tkinter.simpledialog import Dialog
     import tkinter.messagebox as tkMessageBox
     from configparser import ConfigParser
+    import tkinter.tix as Tix
+    import tkinter.font as tkFont
+
+# Python 2.4
+try:
+    all
+except NameError:
+    def all(iterable):
+        for element in iterable:
+            if not element:
+                return False
+        return True
 
 from libtovid import cli
 from libtovid.util import imagemagick_fonts
@@ -58,7 +76,7 @@ def exit_with_traceback(error_message):
 
 def ensure_type(message, required_type, *objects):
     """Ensure that the given objects are of the required type.
-    If not, print a message and exit_with_traceback.
+    If not, print a message and call `exit_with_traceback`.
     """
     for obj in objects:
         if not isinstance(obj, required_type):
@@ -81,11 +99,11 @@ def divide_list(items, pieces):
 
 
 def askyesno(title=None, message=None, **options):
-    """Show a popup with Yes / No options. Return True if Yes was clicked,
-    False if No was clicked.
+    """Show a popup with Yes / No options. Return ``True`` if Yes was clicked,
+    ``False`` if No was clicked.
 
     This function is to work around a bug in Tkinter 8.5 / Python 2.6, where
-    the askyesno function always returns False after a file dialog has been
+    the askyesno function always returns ``False`` after a file dialog has been
     displayed.
     """
     result = tkMessageBox._show(title, message,
@@ -97,6 +115,87 @@ def askyesno(title=None, message=None, **options):
         return False
 
 
+def get_photo_image(filename, width=0, height=0, background='', dither=False):
+    """Get a ``tk.PhotoImage`` from a given image file.
+
+        filename
+            Full path to the image file, in any format that 'convert'
+            understands
+        width
+            Desired image width in pixels
+        height
+            Desired image height in pixels
+        background
+            An '#RRGGBB' string for the desired background color.
+            Only effective for images with transparency.
+        dither
+            True to dither the image. May increase graininess.
+            Smallish images usually look better without dithering.
+
+    Width and height may be used to scale the image in different ways:
+
+        width == 0, height == 0
+            Preserve the original image's size
+        width > 0, height == 0
+            Resize to the given width; height automatically
+            adjusts to maintain aspect ratio
+        width == 0, height > 0
+            Resize to the given height; width automatically
+            adjusts to maintain aspect ratio
+        width > 0, height > 0
+            Resize to exactly the given dimensions
+    """
+    # Convert the image file to gif using 'convert'
+    cmd = cli.Command('convert')
+
+    # Pre-processing
+    # Dither if requested, otherwise disable
+    if dither:
+        cmd.add('-dither')
+    else:
+        cmd.add('+dither')
+    # Set the background color, if provided
+    if background:
+        cmd.add('-background', background)
+
+    cmd.add(filename)
+
+    # Post-processing
+    # Scale appropriately
+    if width > 0 and height == 0:
+        cmd.add('-resize', '%dx' % width)
+    elif width == 0 and height > 0:
+        cmd.add('-resize', 'x%d' % height)
+    elif width > 0 and height > 0:
+        cmd.add('-resize', '%dx%d!' % (width, height))
+
+    # If background color was set, flatten the image
+    if background:
+        cmd.add('-flatten')
+
+    # Convert to gif data
+    cmd.add('gif:-')
+
+    # Run the command, capturing the gif data from standard output
+    cmd.run(capture=True, silent=True)
+    gif_data = cmd.get_output()
+    # Create and return the PhotoImage from the gif data
+    return tk.PhotoImage(data=base64.b64encode(gif_data))
+
+
+def show_icons(window, image):
+    '''Show window manager icons for window, if supported.
+    The icon argument is full path to the image to be used.
+    '''
+    icon = get_photo_image(image, 32, 32)
+    try:
+        window.tk.call('wm', 'iconphoto', window._w, icon)
+    except tk.TclError:
+        print("Could not set window manager icons.  Maybe your "
+              "Tkinter (Tcl) is too old? Continuing anyway.")
+
+
+
 ### --------------------------------------------------------------------
 ### Classes
 ### --------------------------------------------------------------------
@@ -104,12 +203,12 @@ def askyesno(title=None, message=None, **options):
 class ScrollList (tk.Frame):
     """A Listbox with a scrollbar.
 
-    Similar to a tk.Listbox, a ScrollList shows a list of items. tk.Variables
-    may be associated with both the list of items, and the one that is currently
-    selected.
+    Similar to a ``tk.Listbox``, a ScrollList shows a list of items.
+    ``tk.Variable`` objects may be associated with both the list of items, and
+    the one that is currently selected.
     """
     def __init__(self, master=None, items=None,
-                 selected=None):
+                 selected=None, **kwargs):
         """Create a ScrollList widget.
 
             master
@@ -118,6 +217,13 @@ class ScrollList (tk.Frame):
                 ListVar or Python list of items to show in listbox
             selected
                 Tk StringVar to store currently selected choice in
+
+        Keyword arguments:
+
+            width
+                Width of the listbox, in characters
+            height
+                Height of the listbox, in characters
         """
         tk.Frame.__init__(self, master)
 
@@ -132,7 +238,8 @@ class ScrollList (tk.Frame):
         self.scrollbar = tk.Scrollbar(self, orient='vertical',
                                       command=self.scroll)
         self.listbox = tk.Listbox(self, listvariable=self.items,
-                                  width=30, height=7,
+                                  width=kwargs.get('width', 45),
+                                  height=kwargs.get('height', 7),
                                   yscrollcommand=self.scrollbar.set,
                                   exportselection=0)
         self.listbox.pack(side='left', fill='both', expand=True)
@@ -185,9 +292,9 @@ class ScrollList (tk.Frame):
 
 
     def delete(self, first, last=None):
-        """Delete values in a given index range (first, last), not including
-        last itself. If last is None, delete only the item at first index.
-        Calls ``remove`` callbacks for each removed item.
+        """Delete values in a given index range (``first``, ``last``), not
+        including ``last`` itself. If ``last`` is ``None``, delete only the
+        item at index ``first``.  Calls ``remove`` callbacks for each removed item.
         """
         if first == last:
             return
@@ -201,7 +308,7 @@ class ScrollList (tk.Frame):
 
 
     def swap(self, index_a, index_b):
-        """Swap the element at index_a with the one at index_b.
+        """Swap the element at ``index_a`` with the one at ``index_b``.
         Calls ``swap`` callbacks for the swapped items.
         """
         # Use a temporary list, so self.items is only modified once
@@ -233,7 +340,7 @@ class ScrollList (tk.Frame):
 
 
     def select_index(self, index, select_in_linked=True):
-        """Select (highlight) the list item at the given index.
+        """Select (highlight) the list item at the given ``index``.
         Calls ``select`` callbacks for the selected item.
         """
         item_count = self.items.count()
@@ -288,11 +395,11 @@ class ScrollList (tk.Frame):
             action
                 May be 'insert', 'remove', 'select', or 'swap'
             function
-                Callback function, taking (index, value) arguments
-                index is 0-based; passes -1 for the last item
+                Callback function to call for the given action
 
-        All callback functions (except 'swap') must take (index, value)
-        parameters. The 'swap' callback should accept (index_a, index_b).
+        All callback functions except 'swap' must take ``(index, value)``
+        parameters; the 'swap' callback should accept ``(index_a, index_b)``.
+        In all cases, ``index`` is 0-based, and -1 is passed for the last item.
         """
         if action not in ['insert', 'remove', 'select', 'swap']:
             raise ValueError("List callback action must be"
@@ -303,8 +410,8 @@ class ScrollList (tk.Frame):
 
 
     def summon_callbacks(self, action, index, item):
-        """Summon callbacks for the given action,
-        passing index and item to each.
+        """Summon callbacks for the given ``action``, passing ``index`` and
+        ``item`` to each.
         """
         if action not in ['insert', 'remove', 'select', 'swap']:
             raise ValueError("Callback action must be"
@@ -365,10 +472,9 @@ class DragList (ScrollList):
 
 
 class ComboBox (tk.Frame):
-    """A dropdown menu with several choices.
-    """
     def __init__(self, master, choices=None,
-                 variable=None, command=None):
+                 variable=None, command=None,
+                 **kwargs):
         """Create a ComboBox.
 
             master
@@ -381,96 +487,28 @@ class ComboBox (tk.Frame):
                 Function to call when an item in the list is selected
         """
         tk.Frame.__init__(self, master)
-        if type(choices) == list:
-            choices = ListVar(self, choices)
-        self.choices = choices or ListVar()
         self.variable = variable or tk.StringVar()
-        self.command = command
-        self.curindex = 0
-
-        # Text and button
-        # Fit to width of longest choice
-        _width = max([len(str(c)) for c in self.choices.get()])
-        self.text = tk.Entry(self, textvariable=self.variable, width=_width)
-        self.text.pack(side='left', expand=True, fill='both')
-        self.button = tk.Button(self, text="...", command=self.open)
-        self.button.pack(side='left')
-
-        # Dropdown list, displayed when button is clicked
-        self.dropdown = tk.Toplevel(self)
-        # Don't draw window manager frame around the dropdown
-        self.dropdown.wm_overrideredirect(1)
-        # Hide until later
-        self.dropdown.withdraw()
-
-        # List of choices
-        self.chooser = tk.Listbox(self.dropdown, background='#ffffff',
-                                  listvariable=self.choices, width=_width,
-                                  height=self.choices.count())
-        # Use alternating white/gray for choices
-        #for index in range(0, len(self.choices), 2):
-        #    self.chooser.itemconfig(index, bg='LightGray')
-        self.chooser.bind('<Button-1>', self.choose, '+')
-        self.bind_all('<Button>', self.close, '+')
-        self.chooser.bind('<Motion>', self.highlight)
-        self.chooser.grid()
-
-
-    def open(self):
-        """Open/close a panel showing the list of choices.
-        """
-        if self.dropdown.winfo_viewable():
-            self.dropdown.withdraw()
-        else:
-            # Align dropdown list with the text box
-            x = self.text.winfo_rootx()
-            y = self.text.winfo_rooty()
-            self.dropdown.wm_geometry("+%d+%d" % (x, y))
-            # Show list
-            self.dropdown.deiconify()
-
-        # Highlight the current selection
-        self.chooser.itemconfig(self.curindex, background='LightGray')
-
-
-    def close(self, event=None):
-        """Close the panel showing the list of choices.
-        """
-        try:
-            if self.dropdown.winfo_viewable():
-                self.dropdown.withdraw()
-        except tk.TclError:
-            pass
-
-
-    def highlight(self, event=None):
-        """Event handler to highlight an entry on mouse-over.
-        """
-        for index in range(self.choices.count()):
-            self.chooser.itemconfig(index, background='White')
-        index = self.chooser.nearest(event.y)
-        self.chooser.itemconfig(index, background='LightGray')
-
-
-    def choose(self, event=None):
-        """Make a selection from the list, and set the variable.
-        """
-        self.curindex = self.chooser.nearest(event.y)
-        self.variable.set(self.chooser.get(self.curindex))
-        self.close()
-        # Callback, if any
-        if self.command:
-            self.command()
+        # Create and pack the Tix ComboBox
+        self.combo = Tix.ComboBox(self, editable=True, **kwargs)
+        self.combo.config(variable=self.variable, command=command)
+        self.combo.pack(side='left', expand=True, fill='both')
+        # Make the entry box background white
+        # FIXME: This doesn't do anything when editable=False
+        self.combo.subwidget('entry').config(bg='#ffffff')
+        # Add choices, if specified
+        if choices:
+            for index, choice in enumerate(choices):
+                self.combo.insert(index, choice)
 
 
 class FontChooser (Dialog):
-    """A widget for choosing a font.
+    """A widget for choosing an ImageMagick font name.
     """
     # Cache of PhotoImage previews, indexed by font name
     _cache = {}
 
     def __init__(self, parent=None):
-        Dialog.__init__(self, parent, "Font chooser")
+        Dialog.__init__(self, parent, "ImageMagick font chooser")
         # Defined in body()
         self.fontlist = None
         self.preview = None
@@ -517,7 +555,7 @@ class FontChooser (Dialog):
 
 
     def render_font(self, fontname):
-        """Return a tk.PhotoImage preview of the given font.
+        """Return a `tk.PhotoImage` preview of the given font.
         """
         cmd = cli.Command('convert')
         cmd.add('-size',  '500x60')
@@ -588,7 +626,8 @@ class Style:
         root.option_clear()
         # Font
         root.option_add("*font", self.font)
-        root.option_add("*Text.font", ('Courier', 10, 'normal'))
+        #root.option_add("*Text.font", ('Courier', 10, 'normal'))
+        root.option_add("*Text.font", self.font)
         # Background color
         root.option_add("*Scale.troughColor", self.bgcolor)
         root.option_add("*Spinbox.background", self.bgcolor)
@@ -652,7 +691,7 @@ class ConfigWindow (Dialog):
         """Create and display a configuration window.
 
             inifile
-                An .ini-style file to load/save settings from
+                An ``.ini``-style file to load/save settings from
         """
         self.style = style or Style()
         Dialog.__init__(self, master, "Configuration")
@@ -667,40 +706,65 @@ class ConfigWindow (Dialog):
         have the initial focus. Called by the Dialog base class constructor.
         """
         # Font family
+        families = sorted(tkFont.families())
         tk.Label(master, text="Font family").pack(side='top')
-        self.fontfamily = ComboBox(master,
-                                   choices=['Helvetica', 'Courier', 'Times'])
+        self.fontfamily = ScrollList(master, families)
+        self.fontfamily.callback('select', self.refresh)
         self.fontfamily.pack(side='top', fill='both', expand=True)
-        # Font size
-        tk.Label(master, text="Font size").pack(side='top')
-        self.fontsize = ComboBox(master, choices=[8, 10, 12, 15, 18, 24])
-        self.fontsize.pack(side='top')
         # Font style
         tk.Label(master, text="Font style").pack(side='top')
-        self.fontstyle = ComboBox(master, choices=['normal', 'bold'])
+        self.fontstyle = ScrollList(master, ['normal', 'bold'], height=2)
+        self.fontstyle.callback('select', self.refresh)
         self.fontstyle.pack(side='top')
+        # Font size
+        tk.Label(master, text="Font size").pack(side='top')
+        size_var = tk.StringVar()
+        self.fontsize = tk.Spinbox(master, from_=8, to=24,
+           textvariable=size_var, command=self.refresh)
+        self.fontsize.variable = size_var
+        self.fontsize.pack(side='top')
+        # Preview area
+        preview_frame = tk.LabelFrame(master, text="Preview")
+        preview_frame.pack(side='top', fill='both')
+        self.preview = tk.Label(preview_frame,
+            text="The quick brown fox")
+        self.preview.pack(side='top')
         # Use initial values loaded from .ini file
         family, size, style = self.style.font
-        self.fontfamily.variable.set(family)
+        self.fontfamily.selected.set(family)
         self.fontsize.variable.set(size)
-        self.fontstyle.variable.set(style)
+        self.fontstyle.selected.set(style)
+        # Refresh the preview
+        self.refresh()
         # Return widget with initial focus
         return self.fontfamily
 
 
+    def get_font(self):
+        """Return a ``(family, size, style)`` tuple for the current font.
+        """
+        return (self.fontfamily.selected.get(),
+                self.fontsize.variable.get(),
+                self.fontstyle.selected.get())
+
+
+    def refresh(self, *args):
+        """Refresh the preview to show the current font.
+        """
+        self.preview.config(font=self.get_font())
+
+
     def apply(self):
         """Apply the selected configuration settings.
         """
-        self.style.font = (self.fontfamily.variable.get(),
-                           self.fontsize.variable.get(),
-                           self.fontstyle.variable.get())
+        self.style.font = self.get_font()
         self.result = self.style
 
 
 class ScrolledWindow (tk.Tk):
     """A top-level window with scrollbars.
 
-    To use as a container for other widgets, do:
+    To use as a container for other widgets, do::
 
         window = ScrolledWindow()
         button = tk.Button(window.frame, text="Click me", ...)
@@ -740,3 +804,115 @@ class ScrolledWindow (tk.Tk):
         self.canvas.configure(xscrollcommand=h_scroll.set,
                               yscrollcommand=v_scroll.set)
 
+
+class PrettyLabel (tk.Text):
+    """Like a Label, with automatic word-wrapping of paragraphs, and inference
+    of headings and preformatted text from basic markup cues.
+
+    """
+    def __init__(self, master, text, font=None, width=65, height=24):
+        """Create a pretty label for displaying the given text.
+        """
+        # Text widget with the same background color as the master
+        tk.Text.__init__(self, master, wrap='word', borderwidth=0,
+           padx=10, width=width, height=height, exportselection=0)
+        if font:
+            self.config(font=font)
+        # Add the text with formatting applied
+        self.set_text(text)
+        # background and unfocused border color same as master's background
+        bg = master.cget('background')
+        self.config(background=bg, highlightbackground=bg)
+        # Make the text widget read-only
+        self.config(state='disabled')
+
+
+    def rewrap(self, text):
+        """Rewrap the given text, by joining regular paragraphs into
+        single lines, and leaving preformatted text and headings alone.
+        """
+        # Start by splitting into chunks at double-line-breaks
+        lines = []
+        chunks = text.strip().split('\n\n')
+        for chunk in chunks:
+            # If this chunk contains newlines, they could be in paragraphs
+            # (which should wrap) or preformatted text (which should not be
+            # wrapped)
+            if '\n' in chunk:
+                parts = chunk.split('\n')
+                # Preformatted - append each line individually
+                if all(line.startswith(' ') for line in parts):
+                    lines.extend(parts)
+                # Paragraph - join lines with spaces
+                else:
+                    lines.append(' '.join(parts))
+            # No newline in chunk -- It's either a heading or a
+            # one-line paragraph, so append it as-is
+            else:
+                lines.append(chunk)
+            # Include an empty line where the \n\n was before
+            lines.append('')
+        return lines
+
+
+    def set_text(self, text):
+        """Set the widget to contain the given text, with formatting applied.
+        """
+        heading_re = re.compile('^[ A-Z()]+$')
+        pre_re = re.compile('^ +(.+)$')
+
+        # Parse each logical (wrapped) line in the text
+        lineno = 1
+        for line in self.rewrap(text):
+            L = str(lineno)
+            # Heading line
+            if heading_re.match(line):
+                # Capitalize each word in the heading
+                heading = ' '.join(word.capitalize() for word in line.split(' '))
+                self.insert('end', heading)
+                # Unique tag name for this heading
+                tag = 'heading' + L
+                self.tag_add(tag, L+'.0', L+'.end')
+                self.tag_config(tag, font=self.get_heading_font())
+            # Preformatted line
+            elif pre_re.match(line):
+                self.insert('end', line)
+                # Unique tag name for this line
+                tag = 'pre' + L
+                self.tag_add(tag, L+'.0', L+'.end')
+                self.tag_config(tag, font=self.get_monospace_font())
+            # Paragraph
+            else:
+                self.insert('end', line)
+            # Newline to separate each logical line
+            self.insert('end', '\n')
+            lineno += 1
+
+
+    def get_heading_font(self):
+        """Return a bold font, in the current font family.
+        """
+        # The config('font') method returns a 5-tuple, where the last
+        # item is the font descriptor tuple, or 'TkFixedFont' as a default
+        font = self.config('font')[-1]
+        # If font is (family, height, style), increase size and make it bold
+        if isinstance(font, tuple) and len(font) == 3:
+            family, height, style = font
+            return (family, int(height)+6, 'bold')
+        # Default font
+        else:
+            return ('Helvetica', 16, 'bold')
+
+
+    def get_monospace_font(self):
+        """Return a monospace font, with the current font size and style.
+        """
+        font = self.config('font')[-1]
+        # If font is (family, height, style), change family to Courier
+        if isinstance(font, tuple) and len(font) == 3:
+            family, height, style = font
+            return ('Courier', int(height), style)
+        # Default font
+        else:
+            return ('Courier', 12, 'normal')
+
